#!/bin/bash

RUN_TYPES=( "0" "A" "B" )

for RUN_TYPE in ${RUN_TYPES[*]}
do
  CONFNAME="run.restart${RUN_TYPE}.conf"

  case ${RUN_TYPE} in
    "0" )
      TIME_STARTDATE="0000, 1, 1, 0, 0, 0"
      TIME_DURATION="8.D0"
      RESTART_RUN=".false."
      RESTART_IN_BASENAME="init_00000101-000000.000"
    ;;
    "A" )
      TIME_STARTDATE="0000, 1, 1, 0, 0, 0"
      TIME_DURATION="4.D0"
      RESTART_RUN=".false."
      RESTART_IN_BASENAME="init_00000101-000000.000"
    ;;
    "B" )
      TIME_STARTDATE="0000, 1, 1, 0, 0, 4"
      TIME_DURATION="4.D0"
      RESTART_RUN=".true."
      RESTART_IN_BASENAME="restartA_00000101-000004.000"
    ;;
  esac

  cat << EOF > ${CONFNAME}
#####
#
# SCALE-RM run configulation
#
#####

&PARAM_IO
 IO_LOG_BASENAME = "LOG${RUN_TYPE}",
/

&PARAM_PRC
 PRC_NUM_X = 1,
 PRC_NUM_Y = 1,
/

&PARAM_INDEX
 KMAX = 40,
 IMAX =  2,
 JMAX =  2,
 IHALO = 2,
 JHALO = 2,
/

&PARAM_LAND_INDEX
 LKMAX = 5,
/

&PARAM_URBAN_INDEX
 UKMAX = 5,
/

&PARAM_GRID
 DZ = 500.D0,
 DX = 500.D0,
 DY = 500.D0,
/

&PARAM_LAND_GRID
 LDZ = 0.05D0, 0.15D0, 0.30D0, 0.50D0, 1.00D0,
/

&PARAM_URBAN_GRID
 UDZ = 0.01D0, 0.01D0, 0.03D0, 0.05D0, 0.10D0,
/

&PARAM_TIME
 TIME_STARTDATE             = ${TIME_STARTDATE},
 TIME_STARTMS               = 0.D0,
 TIME_DURATION              = ${TIME_DURATION},
 TIME_DURATION_UNIT         = "SEC",
 TIME_DT                    = 2.D0,
 TIME_DT_UNIT               = "SEC",
 TIME_DT_ATMOS_DYN          = 1.D0,
 TIME_DT_ATMOS_DYN_UNIT     = "SEC",
 TIME_DT_ATMOS_PHY_MP       = 2.D0,
 TIME_DT_ATMOS_PHY_MP_UNIT  = "SEC",
 TIME_DT_ATMOS_PHY_RD       = 4.D0,
 TIME_DT_ATMOS_PHY_RD_UNIT  = "SEC",
 TIME_DT_ATMOS_PHY_TB       = 2.D0,
 TIME_DT_ATMOS_PHY_TB_UNIT  = "SEC",
 TIME_DT_OCEAN              = 4.D0,
 TIME_DT_OCEAN_UNIT         = "SEC",
 TIME_DT_LAND               = 4.D0,
 TIME_DT_LAND_UNIT          = "SEC",
 TIME_DT_URBAN              = 4.D0,
 TIME_DT_URBAN_UNIT         = "SEC",
/

&PARAM_MAPPROJ
 MPRJ_basepoint_lon = 180.D0,
 MPRJ_basepoint_lat = 0.D0,
/

&PARAM_LANDUSE
 LANDUSE_MosaicWorld = .true.,
/

&PARAM_RESTART
 RESTART_RUN          = ${RESTART_RUN},
 RESTART_IN_BASENAME  = "${RESTART_IN_BASENAME}",
 RESTART_OUTPUT       = .true.,
 RESTART_OUT_BASENAME = "restart${RUN_TYPE}",
/

&PARAM_STATISTICS
 STATISTICS_checktotal     = .false.,
 STATISTICS_use_globalcomm = .true.,
/

&PARAM_TRACER
 TRACER_TYPE = "TOMITA08",
/

&PARAM_ATMOS
 ATMOS_DYN_TYPE    = "HEVI",
 ATMOS_PHY_MP_TYPE = "TOMITA08",
 ATMOS_PHY_RD_TYPE = "MSTRNX",
 ATMOS_PHY_SF_TYPE = "COUPLE",
 ATMOS_PHY_TB_TYPE = "MYNN",
/

&PARAM_ATMOS_VARS
 ATMOS_VARS_CHECKRANGE = .true.,
/

&PARAM_ATMOS_REFSTATE
 ATMOS_REFSTATE_TYPE        = "INIT",
 ATMOS_REFSTATE_UPDATE_FLAG = .true.,
 ATMOS_REFSTATE_UPDATE_DT   = 2.0D0,
/

&PARAM_ATMOS_BOUNDARY
 ATMOS_BOUNDARY_TYPE       = "CONST",
 ATMOS_BOUNDARY_USE_VELZ   = .true.,
 ATMOS_BOUNDARY_VALUE_VELZ =  0.D0,
 ATMOS_BOUNDARY_TAUZ       = 10.D0,
/

&PARAM_ATMOS_SOLARINS
 ATMOS_SOLARINS_fixedlatlon  = .true.,
/

&PARAM_ATMOS_DYN
 ATMOS_DYN_TINTEG_SHORT_TYPE          = "RK4",
 ATMOS_DYN_TINTEG_TRACER_TYPE         = "RK3WS2002",
 ATMOS_DYN_FVM_FLUX_TYPE              = "CD4",
 ATMOS_DYN_FVM_FLUX_TRACER_TYPE       = "UD3KOREN1993",
 ATMOS_DYN_FLAG_FCT_TRACER            = .false.,
 ATMOS_DYN_NUMERICAL_DIFF_COEF        = 1.D-4,
 ATMOS_DYN_NUMERICAL_DIFF_COEF_TRACER = 0.D0,
/

&PARAM_ATMOS_PHY_RD_MSTRN
 ATMOS_PHY_RD_MSTRN_KADD                  = 30,
 ATMOS_PHY_RD_MSTRN_GASPARA_IN_FILENAME   = "PARAG.29",
 ATMOS_PHY_RD_MSTRN_AEROPARA_IN_FILENAME  = "PARAPC.29",
 ATMOS_PHY_RD_MSTRN_HYGROPARA_IN_FILENAME = "VARDATA.RM29",
 ATMOS_PHY_RD_MSTRN_NBAND                 = 29,
/

&PARAM_ATMOS_PHY_RD_PROFILE
 ATMOS_PHY_RD_PROFILE_TOA                   = 100.D0,
 ATMOS_PHY_RD_PROFILE_CIRA86_IN_FILENAME    = "cira.nc",
 ATMOS_PHY_RD_PROFILE_MIPAS2001_IN_BASENAME = ".",
 DEBUG                                      = .true.,
/

&PARAM_OCEAN
 OCEAN_TYPE = "CONST",
/

&PARAM_OCEAN_PHY_SLAB
 OCEAN_PHY_SLAB_DEPTH = 5.D0,
/

&PARAM_LAND
 LAND_TYPE = "SLAB",
/

&PARAM_LAND_PHY_SLAB
 LAND_PHY_UPDATE_BOTTOM_TEMP  = .false.,
 LAND_PHY_UPDATE_BOTTOM_WATER = .true.,
/

&PARAM_LAND_PROPERTY
 LAND_PROPERTY_IN_FILENAME = "param.bucket.conf",
/

&PARAM_URBAN
 URBAN_TYPE = "SLC",
/

&PARAM_URBAN_PHY_SLC
 STRGR = 0.24D0,
 STRGB = 0.009D0,
 STRGG = 0.24D0,
/
EOF
done
