!-------------------------------------------------------------------------------
!> module ATMOSPHERE / Physics Cloud Microphysics
!!
!! @par Description
!!         Cloud Microphysics
!!
!! @author Team SCALE
!!
!! @par History
!! @li      2013-12-06 (S.Nishizawa) [new]
!! @li      2015-09-08 (Y.Sato)      [mod] Add evaporated cloud number concentration
!!
!<
! Warning: This file was generated from atmos-physics/microphysics/scale_atmos_phy_mp.F90.erb.
!          Do not edit this file.
!-------------------------------------------------------------------------------
module scale_atmos_phy_mp
  !-----------------------------------------------------------------------------
  !
  !++ used modules
  !
  use scale_precision
  use scale_stdio
  use scale_prof
  use scale_grid_index
  use scale_tracer
  !-----------------------------------------------------------------------------
  implicit none
  private
  !-----------------------------------------------------------------------------
  !
  !++ Public procedure
  !
  public :: ATMOS_PHY_MP_setup

  !-----------------------------------------------------------------------------
  !
  !++ Public parameters & variables
  !
  !-----------------------------------------------------------------------------
  !
  !++ Private procedure
  !
  !-----------------------------------------------------------------------------
  !
  !++ Private parameters & variables
  !
  abstract interface
     subroutine mp( &
          DENS,      &
          MOMZ,      &
          MOMX,      &
          MOMY,      &
          RHOT,      &
          QTRC,      &
          CCN,       &
          EVAPORATE, &
          SFLX_rain, &
          SFLX_snow  )
       use scale_precision
       use scale_grid_index
       use scale_tracer
       real(RP), intent(inout) :: DENS(KA,IA,JA)
       real(RP), intent(inout) :: MOMZ(KA,IA,JA)
       real(RP), intent(inout) :: MOMX(KA,IA,JA)
       real(RP), intent(inout) :: MOMY(KA,IA,JA)
       real(RP), intent(inout) :: RHOT(KA,IA,JA)
       real(RP), intent(inout) :: QTRC(KA,IA,JA,QA)
       real(RP), intent(in)    :: CCN(KA,IA,JA)
       real(RP), intent(out)   :: EVAPORATE(KA,IA,JA)
       real(RP), intent(out)   :: SFLX_rain(IA,JA)
       real(RP), intent(out)   :: SFLX_snow(IA,JA)
     end subroutine mp
     subroutine cf( &
          cldfrac, &
          QTRC     )
       use scale_precision
       use scale_grid_index
       use scale_tracer
       real(RP), intent(out) :: cldfrac(KA,IA,JA)
       real(RP), intent(in)  :: QTRC   (KA,IA,JA,QA)
     end subroutine cf
     subroutine er( &
          Re,    &
          QTRC0, &
          DENS0, &
          TEMP0  )
       use scale_precision
       use scale_grid_index
       use scale_tracer
       real(RP), intent(out) :: Re   (KA,IA,JA,MP_QA) ! effective radius          [cm]
       real(RP), intent(in)  :: QTRC0(KA,IA,JA,QA)    ! tracer mass concentration [kg/kg]
       real(RP), intent(in)  :: DENS0(KA,IA,JA)       ! density                   [kg/m3]
       real(RP), intent(in)  :: TEMP0(KA,IA,JA)       ! temperature               [K]
     end subroutine er
     subroutine mr( &
          Qe,    &
          QTRC0  )
       use scale_precision
       use scale_grid_index
       use scale_tracer
       real(RP), intent(out) :: Qe   (KA,IA,JA,MP_QA) ! mixing ratio of each cateory [kg/kg]
       real(RP), intent(in)  :: QTRC0(KA,IA,JA,QA)    ! tracer mass concentration [kg/kg]
     end subroutine mr
  end interface
  procedure(mp), pointer :: ATMOS_PHY_MP => NULL()
  procedure(cf), pointer :: ATMOS_PHY_MP_CloudFraction => NULL()
  procedure(er), pointer :: ATMOS_PHY_MP_EffectiveRadius => NULL()
  procedure(mr), pointer :: ATMOS_PHY_MP_MixingRatio => NULL()
  public :: ATMOS_PHY_MP
  public :: ATMOS_PHY_MP_CloudFraction
  public :: ATMOS_PHY_MP_EffectiveRadius
  public :: ATMOS_PHY_MP_MixingRatio

  real(RP), pointer, public :: ATMOS_PHY_MP_DENS(:)

  !-----------------------------------------------------------------------------
contains
  !-----------------------------------------------------------------------------
  !> Setup Cloud Microphysics
  !-----------------------------------------------------------------------------
  subroutine ATMOS_PHY_MP_setup( MP_TYPE )
    use scale_process, only: &
       PRC_MPIstop
#define EXTM(pre, name, post) pre ## name ## post
#define NAME(pre, name, post) EXTM(pre, name, post)
#ifdef MP
    use NAME(scale_atmos_phy_mp_, MP,), only: &
       NAME(ATMOS_PHY_MP_, MP, _setup), &
       NAME(ATMOS_PHY_MP_, MP,), &
       NAME(ATMOS_PHY_MP_, MP, _CloudFraction), &
       NAME(ATMOS_PHY_MP_, MP, _EffectiveRadius), &
       NAME(ATMOS_PHY_MP_, MP, _MixingRatio), &
       NAME(ATMOS_PHY_MP_, MP, _DENS) => ATMOS_PHY_MP_DENS
#else
    use scale_atmos_phy_mp_dry, only: &
       ATMOS_PHY_MP_dry_setup, &
       ATMOS_PHY_MP_dry, &
       ATMOS_PHY_MP_dry_CloudFraction, &
       ATMOS_PHY_MP_dry_EffectiveRadius, &
       ATMOS_PHY_MP_dry_MixingRatio, &
       ATMOS_PHY_MP_dry_DENS => ATMOS_PHY_MP_DENS
    use scale_atmos_phy_mp_kessler, only: &
       ATMOS_PHY_MP_kessler_setup, &
       ATMOS_PHY_MP_kessler, &
       ATMOS_PHY_MP_kessler_CloudFraction, &
       ATMOS_PHY_MP_kessler_EffectiveRadius, &
       ATMOS_PHY_MP_kessler_MixingRatio, &
       ATMOS_PHY_MP_kessler_DENS => ATMOS_PHY_MP_DENS
    use scale_atmos_phy_mp_tomita08, only: &
       ATMOS_PHY_MP_tomita08_setup, &
       ATMOS_PHY_MP_tomita08, &
       ATMOS_PHY_MP_tomita08_CloudFraction, &
       ATMOS_PHY_MP_tomita08_EffectiveRadius, &
       ATMOS_PHY_MP_tomita08_MixingRatio, &
       ATMOS_PHY_MP_tomita08_DENS => ATMOS_PHY_MP_DENS
    use scale_atmos_phy_mp_sn14, only: &
       ATMOS_PHY_MP_sn14_setup, &
       ATMOS_PHY_MP_sn14, &
       ATMOS_PHY_MP_sn14_CloudFraction, &
       ATMOS_PHY_MP_sn14_EffectiveRadius, &
       ATMOS_PHY_MP_sn14_MixingRatio, &
       ATMOS_PHY_MP_sn14_DENS => ATMOS_PHY_MP_DENS
    use scale_atmos_phy_mp_suzuki10, only: &
       ATMOS_PHY_MP_suzuki10_setup, &
       ATMOS_PHY_MP_suzuki10, &
       ATMOS_PHY_MP_suzuki10_CloudFraction, &
       ATMOS_PHY_MP_suzuki10_EffectiveRadius, &
       ATMOS_PHY_MP_suzuki10_MixingRatio, &
       ATMOS_PHY_MP_suzuki10_DENS => ATMOS_PHY_MP_DENS
    use scale_atmos_phy_mp_sdm, only: &
       ATMOS_PHY_MP_sdm_setup, &
       ATMOS_PHY_MP_sdm, &
       ATMOS_PHY_MP_sdm_CloudFraction, &
       ATMOS_PHY_MP_sdm_EffectiveRadius, &
       ATMOS_PHY_MP_sdm_MixingRatio, &
       ATMOS_PHY_MP_sdm_DENS => ATMOS_PHY_MP_DENS
#endif
    implicit none

    character(len=*), intent(in) :: MP_TYPE
    !---------------------------------------------------------------------------

#ifdef MP
    call NAME(ATMOS_PHY_MP_, MP, _setup)( MP_TYPE )
    ATMOS_PHY_MP                 => NAME(ATMOS_PHY_MP_, MP,)
    ATMOS_PHY_MP_CloudFraction   => NAME(ATMOS_PHY_MP_, MP, _CloudFraction)
    ATMOS_PHY_MP_EffectiveRadius => NAME(ATMOS_PHY_MP_, MP, _EffectiveRadius)
    ATMOS_PHY_MP_MixingRatio     => NAME(ATMOS_PHY_MP_, MP, _MixingRatio)
    ATMOS_PHY_MP_DENS            => NAME(ATMOS_PHY_MP_, MP, _DENS)
#else
    select case ( MP_TYPE )
    case ( 'DRY' )
       call ATMOS_PHY_MP_dry_setup( MP_TYPE )
       ATMOS_PHY_MP                 => ATMOS_PHY_MP_dry
       ATMOS_PHY_MP_CloudFraction   => ATMOS_PHY_MP_dry_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_dry_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio     => ATMOS_PHY_MP_dry_MixingRatio
       ATMOS_PHY_MP_DENS            => ATMOS_PHY_MP_dry_DENS
    case ( 'KESSLER' )
       call ATMOS_PHY_MP_kessler_setup( MP_TYPE )
       ATMOS_PHY_MP                 => ATMOS_PHY_MP_kessler
       ATMOS_PHY_MP_CloudFraction   => ATMOS_PHY_MP_kessler_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_kessler_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio     => ATMOS_PHY_MP_kessler_MixingRatio
       ATMOS_PHY_MP_DENS            => ATMOS_PHY_MP_kessler_DENS
    case ( 'TOMITA08' )
       call ATMOS_PHY_MP_tomita08_setup( MP_TYPE )
       ATMOS_PHY_MP                 => ATMOS_PHY_MP_tomita08
       ATMOS_PHY_MP_CloudFraction   => ATMOS_PHY_MP_tomita08_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_tomita08_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio     => ATMOS_PHY_MP_tomita08_MixingRatio
       ATMOS_PHY_MP_DENS            => ATMOS_PHY_MP_tomita08_DENS
    case ( 'SN14' )
       call ATMOS_PHY_MP_sn14_setup( MP_TYPE )
       ATMOS_PHY_MP                 => ATMOS_PHY_MP_sn14
       ATMOS_PHY_MP_CloudFraction   => ATMOS_PHY_MP_sn14_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_sn14_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio     => ATMOS_PHY_MP_sn14_MixingRatio
       ATMOS_PHY_MP_DENS            => ATMOS_PHY_MP_sn14_DENS
    case ( 'SUZUKI10' )
       call ATMOS_PHY_MP_suzuki10_setup( MP_TYPE )
       ATMOS_PHY_MP                 => ATMOS_PHY_MP_suzuki10
       ATMOS_PHY_MP_CloudFraction   => ATMOS_PHY_MP_suzuki10_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_suzuki10_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio     => ATMOS_PHY_MP_suzuki10_MixingRatio
       ATMOS_PHY_MP_DENS            => ATMOS_PHY_MP_suzuki10_DENS
    case ( 'SDM' )
       call ATMOS_PHY_MP_sdm_setup( MP_TYPE )
       ATMOS_PHY_MP                 => ATMOS_PHY_MP_sdm
       ATMOS_PHY_MP_CloudFraction   => ATMOS_PHY_MP_sdm_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_sdm_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio     => ATMOS_PHY_MP_sdm_MixingRatio
       ATMOS_PHY_MP_DENS            => ATMOS_PHY_MP_sdm_DENS
    end select
#endif

    return
  end subroutine ATMOS_PHY_MP_setup

end module scale_atmos_phy_mp
